﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ShadowMapping
{
    public class Camera
    {
        Vector3 front, up, right;
        Vector3 pos;

        Matrix view;

        bool viewDirty = true; // true if view should be recomputed

        public float MoveRate = 2.0f;
        public float RotationRate = 0.01f;

        public Camera(Vector3 position, Vector3 front)
        {
            this.pos = position;
            this.front = front;
            this.front.Normalize();
            this.up = Vector3.UnitY;
            this.right = Vector3.Cross(front, up);
        }

        private void RecalcViewMatrix()
        {
            view = Matrix.CreateLookAt(pos, pos + front, up);
            viewDirty = false;
        }

        public void Move(float toFront, float toRight)
        {
            pos += MoveRate * (toFront * front + toRight * right);
            viewDirty = true;
        }

        public void Rotate(float upDown, float rightLeft)
        {
            front = Vector3.Transform(front,
                Matrix.CreateFromAxisAngle(right, RotationRate * upDown) *
                Matrix.CreateFromAxisAngle(up, RotationRate * rightLeft));
            this.right = Vector3.Cross(front, up);
            viewDirty = true;
        }

        #region Accessors
        public Vector3 Front
        {
            get { return front; }
            set
            {
                front = value;
                front.Normalize();
                right = Vector3.Cross(front, up);
                viewDirty = true;
            }
        }

        public Vector3 Position
        {
            get { return pos; }
            set
            {
                pos = value;
                viewDirty = true;
            }
        }


        public Matrix View
        {
            get
            {
                if (viewDirty)
                    RecalcViewMatrix();
                return view;
            }
        }
        #endregion
    }
}
