shared float4x4 World;
shared float4x4 View;
shared float4x4 Projection;

shared float4x4 LightViewProjection;


shared float3 cameraPosition;

//light properties
float3 lightPosition;
float3 lightDirection;
float cosInnerCone;
float cosOutterCone;
float4 ambientLightColor;
float4 diffuseLightColor;
float4 specularLightColor;

//material properties
shared float specularPower;
shared float specularIntensity;

struct VertexShaderInput
{
    float4 Position : POSITION0;
	float3 Normal : NORMAL;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
	float3 Normal : TEXCOORD0;
    float3 WorldPosition : TEXCOORD1;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
	output.WorldPosition = worldPosition;
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	float3 normal = mul(float4(input.Normal, 0), World);
	output.Normal = normalize(normal);	

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
	float4 diffuse = 0;
	float4 specular = 0;

	float3 directionToLight = normalize(lightPosition - input.WorldPosition);
	float diffuseIntensity = saturate( dot(directionToLight, input.Normal));
	float spotFactor = smoothstep(cosOutterCone, cosInnerCone, dot(directionToLight, -lightDirection));
	diffuse = spotFactor * diffuseLightColor * diffuseIntensity;

	//calculate Phong components per-pixel
	float3 reflectionVector = normalize(reflect(-directionToLight, input.Normal));
	float3 directionToCamera = normalize(cameraPosition - input.WorldPosition);

	//calculate specular component
	specular = specularLightColor * specularIntensity * 
				   pow(saturate(dot(reflectionVector, directionToCamera)), 
					   specularPower);

	//all color components are summed in the pixel shader
	float4 Color = specular  + diffuse + ambientLightColor;
	Color.a = 1.0;
	
	return Color;
}

technique DirectionalLight
{
    pass Pass1
    {
        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}